@@ -67,7 +67,7 @@ public class CoyoteAdapter implements Adapter {
 
     // -------------------------------------------------------------- Constants
 
-    private static final String POWERED_BY = "Servlet/6.0 JSP/3.1 " + "(" + ServerInfo.getServerInfo() + " Java/" +
+    private static final String POWERED_BY = "Servlet/6.1 JSP/4.0 " + "(" + ServerInfo.getServerInfo() + " Java/" +
             System.getProperty("java.vm.vendor") + "/" + System.getProperty("java.runtime.version") + ")";
 
     private static final EnumSet<SessionTrackingMode> SSL_ONLY = EnumSet.of(SessionTrackingMode.SSL);
@@ -144,7 +144,7 @@ public class CoyoteAdapter implements Adapter {
                 Context context = request.getContext();
                 ClassLoader oldCL = null;
                 try {
-                    oldCL = context.bind(false, null);
+                    oldCL = context.bind(null);
                     if (req.getReadListener() != null) {
                         req.getReadListener().onError(t);
                     }
@@ -154,7 +154,7 @@ public class CoyoteAdapter implements Adapter {
                     res.action(ActionCode.CLOSE_NOW, t);
                     asyncConImpl.setErrorState(t, true);
                 } finally {
-                    context.unbind(false, oldCL);
+                    context.unbind(oldCL);
                 }
             }
 
@@ -166,7 +166,7 @@ public class CoyoteAdapter implements Adapter {
                     Context context = request.getContext();
                     ClassLoader oldCL = null;
                     try {
-                        oldCL = context.bind(false, null);
+                        oldCL = context.bind(null);
                         res.onWritePossible();
                         if (request.isFinished() && req.sendAllDataReadEvent() && readListener != null) {
                             readListener.onAllDataRead();
@@ -177,6 +177,8 @@ public class CoyoteAdapter implements Adapter {
                         }
                     } catch (Throwable t) {
                         ExceptionUtils.handleThrowable(t);
+                        // Allow the error handling to write to the response
+                        response.setSuspended(false);
                         // Need to trigger the call to AbstractProcessor.setErrorState()
                         // before the listener is called so the listener can call complete
                         // Therefore no need to set success=false as that would trigger a
@@ -186,13 +188,13 @@ public class CoyoteAdapter implements Adapter {
                         res.action(ActionCode.CLOSE_NOW, t);
                         asyncConImpl.setErrorState(t, true);
                     } finally {
-                        context.unbind(false, oldCL);
+                        context.unbind(oldCL);
                     }
                 } else if (readListener != null && status == SocketEvent.OPEN_READ) {
                     Context context = request.getContext();
                     ClassLoader oldCL = null;
                     try {
-                        oldCL = context.bind(false, null);
+                        oldCL = context.bind(null);
                         // If data is being read on a non-container thread a
                         // dispatch with status OPEN_READ will be used to get
                         // execution back on a container thread for the
@@ -210,6 +212,8 @@ public class CoyoteAdapter implements Adapter {
                         }
                     } catch (Throwable t) {
                         ExceptionUtils.handleThrowable(t);
+                        // Allow the error handling to write to the response
+                        response.setSuspended(false);
                         // Need to trigger the call to AbstractProcessor.setErrorState()
                         // before the listener is called so the listener can call complete
                         // Therefore no need to set success=false as that would trigger a
@@ -219,7 +223,7 @@ public class CoyoteAdapter implements Adapter {
                         res.action(ActionCode.CLOSE_NOW, t);
                         asyncConImpl.setErrorState(t, true);
                     } finally {
-                        context.unbind(false, oldCL);
+                        context.unbind(oldCL);
                     }
                 }
             }
@@ -233,8 +237,8 @@ public class CoyoteAdapter implements Adapter {
 
             if (request.isAsyncDispatching()) {
                 connector.getService().getContainer().getPipeline().getFirst().invoke(request, response);
-                Throwable t = (Throwable) request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
-                if (t != null) {
+                if (response.isError()) {
+                    Throwable t = (Throwable) request.getAttribute(RequestDispatcher.ERROR_EXCEPTION);
                     asyncConImpl.setErrorState(t, true);
                 }
             }
@@ -249,10 +253,9 @@ public class CoyoteAdapter implements Adapter {
             AtomicBoolean error = new AtomicBoolean(false);
             res.action(ActionCode.IS_ERROR, error);
             if (error.get()) {
-                if (request.isAsyncCompleting()) {
-                    // Connection will be forcibly closed which will prevent
-                    // completion happening at the usual point. Need to trigger
-                    // call to onComplete() here.
+                if (request.isAsyncCompleting() || request.isAsyncDispatching()) {
+                    // Connection will be forcibly closed which will prevent completion/dispatch happening at the usual
+                    // point. Trigger post processing here.
                     res.action(ActionCode.ASYNC_POST_PROCESS, null);
                 }
                 success = false;
@@ -304,10 +307,8 @@ public class CoyoteAdapter implements Adapter {
 
         if (request == null) {
             // Create objects
-            request = connector.createRequest();
-            request.setCoyoteRequest(req);
-            response = connector.createResponse();
-            response.setCoyoteResponse(res);
+            request = connector.createRequest(req);
+            response = connector.createResponse(res);
 
             // Link objects
             request.setResponse(response);
@@ -348,12 +349,12 @@ public class CoyoteAdapter implements Adapter {
                     // method so this needs to be checked here
                     ClassLoader oldCL = null;
                     try {
-                        oldCL = request.getContext().bind(false, null);
+                        oldCL = request.getContext().bind(null);
                         if (req.sendAllDataReadEvent()) {
                             req.getReadListener().onAllDataRead();
                         }
                     } finally {
-                        request.getContext().unbind(false, oldCL);
+                        request.getContext().unbind(oldCL);
                     }
                 }
 
@@ -451,10 +452,8 @@ public class CoyoteAdapter implements Adapter {
 
         if (request == null) {
             // Create objects
-            request = connector.createRequest();
-            request.setCoyoteRequest(req);
-            response = connector.createResponse();
-            response.setCoyoteResponse(res);
+            request = connector.createRequest(req);
+            response = connector.createResponse(res);
 
             // Link objects
             request.setResponse(response);
@@ -553,7 +552,6 @@ public class CoyoteAdapter implements Adapter {
      * @throws IOException      If there is insufficient space in a buffer while processing headers
      * @throws ServletException If the supported methods of the target servlet cannot be determined
      */
-    @SuppressWarnings("deprecation")
     protected boolean postParseRequest(org.apache.coyote.Request req, Request request, org.apache.coyote.Response res,
             Response response) throws IOException, ServletException {
 
@@ -592,7 +590,7 @@ public class CoyoteAdapter implements Adapter {
 
         // Check for ping OPTIONS * request
         if (undecodedURI.equals("*")) {
-            if (req.method().equalsIgnoreCase("OPTIONS")) {
+            if (req.method().equals("OPTIONS")) {
                 StringBuilder allow = new StringBuilder();
                 allow.append("GET, HEAD, POST, PUT, DELETE, OPTIONS");
                 // Trace if allowed
@@ -611,7 +609,7 @@ public class CoyoteAdapter implements Adapter {
         MessageBytes decodedURI = req.decodedURI();
 
         // Filter CONNECT method
-        if (req.method().equalsIgnoreCase("CONNECT")) {
+        if (req.method().equals("CONNECT")) {
             response.sendError(HttpServletResponse.SC_NOT_IMPLEMENTED, sm.getString("coyoteAdapter.connect"));
         } else {
             // No URI for CONNECT requests
@@ -632,7 +630,8 @@ public class CoyoteAdapter implements Adapter {
                 // %xx decoding of the URL
                 try {
                     req.getURLDecoder().convert(decodedURI.getByteChunk(),
-                            connector.getEncodedSolidusHandlingInternal());
+                            connector.getEncodedSolidusHandlingInternal(),
+                            connector.getEncodedReverseSolidusHandlingInternal());
                 } catch (IOException ioe) {
                     response.sendError(400, sm.getString("coyoteAdapter.invalidURIWithMessage", ioe.getMessage()));
                 }
@@ -810,7 +809,7 @@ public class CoyoteAdapter implements Adapter {
         }
 
         // Filter TRACE method
-        if (!connector.getAllowTrace() && req.method().equalsIgnoreCase("TRACE")) {
+        if (!connector.getAllowTrace() && req.method().equals("TRACE")) {
             Wrapper wrapper = request.getWrapper();
             String header = null;
             if (wrapper != null) {
@@ -906,10 +905,10 @@ public class CoyoteAdapter implements Adapter {
         // encoding that doesn't give the expected result so be explicit
         Charset charset = connector.getURICharset();
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("coyoteAdapter.debug", "uriBC", uriBC.toString()));
-            log.debug(sm.getString("coyoteAdapter.debug", "semicolon", String.valueOf(semicolon)));
-            log.debug(sm.getString("coyoteAdapter.debug", "enc", charset.name()));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("coyoteAdapter.debug", "uriBC", uriBC.toString()));
+            log.trace(sm.getString("coyoteAdapter.debug", "semicolon", String.valueOf(semicolon)));
+            log.trace(sm.getString("coyoteAdapter.debug", "enc", charset.name()));
         }
 
         while (semicolon > -1) {
@@ -940,10 +939,10 @@ public class CoyoteAdapter implements Adapter {
                 uriBC.setEnd(start + semicolon);
             }
 
-            if (log.isDebugEnabled()) {
-                log.debug(sm.getString("coyoteAdapter.debug", "pathParamStart", String.valueOf(pathParamStart)));
-                log.debug(sm.getString("coyoteAdapter.debug", "pathParamEnd", String.valueOf(pathParamEnd)));
-                log.debug(sm.getString("coyoteAdapter.debug", "pv", pv));
+            if (log.isTraceEnabled()) {
+                log.trace(sm.getString("coyoteAdapter.debug", "pathParamStart", String.valueOf(pathParamStart)));
+                log.trace(sm.getString("coyoteAdapter.debug", "pathParamEnd", String.valueOf(pathParamEnd)));
+                log.trace(sm.getString("coyoteAdapter.debug", "pv", pv));
             }
 
             if (pv != null) {
@@ -952,10 +951,10 @@ public class CoyoteAdapter implements Adapter {
                     String name = pv.substring(0, equals);
                     String value = pv.substring(equals + 1);
                     request.addPathParameter(name, value);
-                    if (log.isDebugEnabled()) {
-                        log.debug(sm.getString("coyoteAdapter.debug", "equals", String.valueOf(equals)));
-                        log.debug(sm.getString("coyoteAdapter.debug", "name", name));
-                        log.debug(sm.getString("coyoteAdapter.debug", "value", value));
+                    if (log.isTraceEnabled()) {
+                        log.trace(sm.getString("coyoteAdapter.debug", "equals", String.valueOf(equals)));
+                        log.trace(sm.getString("coyoteAdapter.debug", "name", name));
+                        log.trace(sm.getString("coyoteAdapter.debug", "value", value));
                     }
                 }
             }
@@ -1019,8 +1018,8 @@ public class CoyoteAdapter implements Adapter {
                     request.setRequestedSessionId(scookie.getValue().toString());
                     request.setRequestedSessionCookie(true);
                     request.setRequestedSessionURL(false);
-                    if (log.isDebugEnabled()) {
-                        log.debug(" Requested cookie session id is " + request.getRequestedSessionId());
+                    if (log.isTraceEnabled()) {
+                        log.trace(" Requested cookie session id is " + request.getRequestedSessionId());
                     }
                 } else {
                     if (!request.isRequestedSessionIdValid()) {
