@@ -24,9 +24,7 @@ import java.io.ObjectOutputStream;
 import java.io.ObjectStreamException;
 import java.io.Serializable;
 import java.io.WriteAbortedException;
-import java.security.AccessController;
 import java.security.Principal;
-import java.security.PrivilegedAction;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Enumeration;
@@ -50,14 +48,12 @@ import jakarta.servlet.http.HttpSessionIdListener;
 import jakarta.servlet.http.HttpSessionListener;
 
 import org.apache.catalina.Context;
-import org.apache.catalina.Globals;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Session;
 import org.apache.catalina.SessionEvent;
 import org.apache.catalina.SessionListener;
 import org.apache.catalina.TomcatPrincipal;
 import org.apache.catalina.authenticator.SavedRequest;
-import org.apache.catalina.security.SecurityUtil;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.res.StringManager;
 
@@ -247,20 +243,12 @@ public class StandardSession implements HttpSession, Session, Serializable {
     // ----------------------------------------------------- Session Properties
 
 
-    /**
-     * Return the authentication type used to authenticate our cached Principal, if any.
-     */
     @Override
     public String getAuthType() {
         return this.authType;
     }
 
 
-    /**
-     * Set the authentication type used to authenticate our cached Principal, if any.
-     *
-     * @param authType The new cached authentication type
-     */
     @Override
     public void setAuthType(String authType) {
         String oldAuthType = this.authType;
@@ -269,12 +257,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Set the creation time for this session. This method is called by the Manager when an existing Session instance is
-     * reused.
-     *
-     * @param time The new creation time
-     */
     @Override
     public void setCreationTime(long time) {
 
@@ -285,38 +267,24 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return the session identifier for this session.
-     */
     @Override
     public String getId() {
         return this.id;
     }
 
 
-    /**
-     * Return the session identifier for this session.
-     */
     @Override
     public String getIdInternal() {
         return this.id;
     }
 
 
-    /**
-     * Set the session identifier for this session.
-     *
-     * @param id The new session identifier
-     */
     @Override
     public void setId(String id) {
         setId(id, true);
     }
 
 
-    /**
-     * {@inheritDoc}
-     */
     @Override
     public void setId(String id, boolean notify) {
 
@@ -342,7 +310,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
     public void tellNew() {
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_CREATED_EVENT, null);
+        fireSessionEvent(SESSION_CREATED_EVENT, null);
 
         // Notify interested application event listeners
         Context context = manager.getContext();
@@ -372,16 +340,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
 
     }
 
-    /**
-     * Inform the listeners about the change session ID.
-     *
-     * @param newId                    new session ID
-     * @param oldId                    old session ID
-     * @param notifySessionListeners   Should any associated sessionListeners be notified that session ID has been
-     *                                     changed?
-     * @param notifyContainerListeners Should any associated ContainerListeners be notified that session ID has been
-     *                                     changed?
-     */
     @Override
     public void tellChangedSessionId(String newId, String oldId, boolean notifySessionListeners,
             boolean notifyContainerListeners) {
@@ -414,11 +372,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return the last time the client sent a request associated with this session, as the number of milliseconds since
-     * midnight, January 1, 1970 GMT. Actions that your application takes, such as getting or setting a value associated
-     * with the session, do not affect the access time. This one gets updated whenever a request starts.
-     */
     @Override
     public long getThisAccessedTime() {
 
@@ -429,21 +382,11 @@ public class StandardSession implements HttpSession, Session, Serializable {
         return this.thisAccessedTime;
     }
 
-    /**
-     * Return the last client access time without invalidation check
-     *
-     * @see #getThisAccessedTime()
-     */
     @Override
     public long getThisAccessedTimeInternal() {
         return this.thisAccessedTime;
     }
 
-    /**
-     * Return the last time the client sent a request associated with this session, as the number of milliseconds since
-     * midnight, January 1, 1970 GMT. Actions that your application takes, such as getting or setting a value associated
-     * with the session, do not affect the access time. This one gets updated whenever a request finishes.
-     */
     @Override
     public long getLastAccessedTime() {
 
@@ -454,19 +397,11 @@ public class StandardSession implements HttpSession, Session, Serializable {
         return this.lastAccessedTime;
     }
 
-    /**
-     * Return the last client access time without invalidation check
-     *
-     * @see #getLastAccessedTime()
-     */
     @Override
     public long getLastAccessedTimeInternal() {
         return this.lastAccessedTime;
     }
 
-    /**
-     * Return the idle time (in milliseconds) from last client access time.
-     */
     @Override
     public long getIdleTime() {
 
@@ -477,11 +412,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
         return getIdleTimeInternal();
     }
 
-    /**
-     * Return the idle time from last client access time without invalidation check
-     *
-     * @see #getIdleTime()
-     */
     @Override
     public long getIdleTimeInternal() {
         long timeNow = System.currentTimeMillis();
@@ -494,78 +424,42 @@ public class StandardSession implements HttpSession, Session, Serializable {
         return timeIdle;
     }
 
-    /**
-     * Return the Manager within which this Session is valid.
-     */
     @Override
     public Manager getManager() {
         return this.manager;
     }
 
 
-    /**
-     * Set the Manager within which this Session is valid.
-     *
-     * @param manager The new Manager
-     */
     @Override
     public void setManager(Manager manager) {
         this.manager = manager;
     }
 
 
-    /**
-     * Return the maximum time interval, in seconds, between client requests before the servlet container will
-     * invalidate the session. A negative time indicates that the session should never time out.
-     */
     @Override
     public int getMaxInactiveInterval() {
         return this.maxInactiveInterval;
     }
 
 
-    /**
-     * Set the maximum time interval, in seconds, between client requests before the servlet container will invalidate
-     * the session. A zero or negative time indicates that the session should never time out.
-     *
-     * @param interval The new maximum interval
-     */
     @Override
     public void setMaxInactiveInterval(int interval) {
         this.maxInactiveInterval = interval;
     }
 
 
-    /**
-     * Set the <code>isNew</code> flag for this session.
-     *
-     * @param isNew The new value for the <code>isNew</code> flag
-     */
     @Override
     public void setNew(boolean isNew) {
         this.isNew = isNew;
     }
 
 
-    /**
-     * Return the authenticated Principal that is associated with this Session. This provides an
-     * <code>Authenticator</code> with a means to cache a previously authenticated Principal, and avoid potentially
-     * expensive <code>Realm.authenticate()</code> calls on every request. If there is no current associated Principal,
-     * return <code>null</code>.
-     */
     @Override
     public Principal getPrincipal() {
         return this.principal;
     }
 
 
-    /**
-     * Set the authenticated Principal that is associated with this Session. This provides an <code>Authenticator</code>
-     * with a means to cache a previously authenticated Principal, and avoid potentially expensive
-     * <code>Realm.authenticate()</code> calls on every request.
-     *
-     * @param principal The new Principal, or <code>null</code> if none
-     */
     @Override
     public void setPrincipal(Principal principal) {
 
@@ -576,25 +470,15 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return the <code>HttpSession</code> for which this object is the facade.
-     */
     @Override
     public HttpSession getSession() {
         if (facade == null) {
-            if (SecurityUtil.isPackageProtectionEnabled()) {
-                facade = AccessController.doPrivileged(new PrivilegedNewSessionFacade(this));
-            } else {
-                facade = new StandardSessionFacade(this);
-            }
+            facade = new StandardSessionFacade(this);
         }
         return facade;
     }
 
 
-    /**
-     * Return the <code>isValid</code> flag for this session.
-     */
     @Override
     public boolean isValid() {
 
@@ -621,24 +505,25 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Set the <code>isValid</code> flag for this session.
-     *
-     * @param isValid The new value for the <code>isValid</code> flag
-     */
     @Override
     public void setValid(boolean isValid) {
         this.isValid = isValid;
     }
 
 
+    @Override
+    public Accessor getAccessor() {
+        if (!isValidInternal()) {
+            throw new IllegalStateException(sm.getString("standardSession.getAccessor.ise"));
+        }
+
+        return new StandardSessionAccessor(getManager(), getId());
+    }
+
+
     // ------------------------------------------------- Session Public Methods
 
 
-    /**
-     * Update the accessed time information for this session. This method should be called by the context when a request
-     * comes in for a particular session, even if the application does not reference it.
-     */
     @Override
     public void access() {
 
@@ -651,9 +536,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * End the access.
-     */
     @Override
     public void endAccess() {
 
@@ -677,9 +559,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Add a session event listener to this component.
-     */
     @Override
     public void addSessionListener(SessionListener listener) {
 
@@ -688,10 +567,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Perform the internal processing required to invalidate this session, without triggering an exception if the
-     * session has already expired.
-     */
     @Override
     public void expire() {
 
@@ -732,7 +607,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
             expiring = true;
 
             // Notify interested application event listeners
-            // FIXME - Assumes we call listeners in reverse order
+            // Call listeners in reverse order
             Context context = manager.getContext();
 
             // The call to expire() may not have been triggered by the webapp.
@@ -741,7 +616,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
             if (notify) {
                 ClassLoader oldContextClassLoader = null;
                 try {
-                    oldContextClassLoader = context.bind(Globals.IS_SECURITY_ENABLED, null);
+                    oldContextClassLoader = context.bind(null);
                     Object listeners[] = context.getApplicationLifecycleListeners();
                     if (listeners != null && listeners.length > 0) {
                         HttpSessionEvent event = new HttpSessionEvent(getSession());
@@ -767,7 +642,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
                         }
                     }
                 } finally {
-                    context.unbind(Globals.IS_SECURITY_ENABLED, oldContextClassLoader);
+                    context.unbind(oldContextClassLoader);
                 }
             }
 
@@ -780,7 +655,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
 
             // Notify interested session event listeners
             if (notify) {
-                fireSessionEvent(Session.SESSION_DESTROYED_EVENT, null);
+                fireSessionEvent(SESSION_DESTROYED_EVENT, null);
             }
 
             // Call the logout method
@@ -801,12 +676,12 @@ public class StandardSession implements HttpSession, Session, Serializable {
             String keys[] = keys();
             ClassLoader oldContextClassLoader = null;
             try {
-                oldContextClassLoader = context.bind(Globals.IS_SECURITY_ENABLED, null);
+                oldContextClassLoader = context.bind(null);
                 for (String key : keys) {
                     removeAttributeInternal(key, notify);
                 }
             } finally {
-                context.unbind(Globals.IS_SECURITY_ENABLED, oldContextClassLoader);
+                context.unbind(oldContextClassLoader);
             }
         }
 
@@ -819,7 +694,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
     public void passivate() {
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_PASSIVATED_EVENT, null);
+        fireSessionEvent(SESSION_PASSIVATED_EVENT, null);
 
         // Notify ActivationListeners
         HttpSessionEvent event = null;
@@ -853,7 +728,7 @@ public class StandardSession implements HttpSession, Session, Serializable {
         }
 
         // Notify interested session event listeners
-        fireSessionEvent(Session.SESSION_ACTIVATED_EVENT, null);
+        fireSessionEvent(SESSION_ACTIVATED_EVENT, null);
 
         // Notify ActivationListeners
         HttpSessionEvent event = null;
@@ -875,30 +750,18 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return the object bound with the specified name to the internal notes for this session, or <code>null</code> if
-     * no such binding exists.
-     *
-     * @param name Name of the note to be returned
-     */
     @Override
     public Object getNote(String name) {
         return notes.get(name);
     }
 
 
-    /**
-     * Return an Iterator containing the String names of all notes bindings that exist for this session.
-     */
     @Override
     public Iterator<String> getNoteNames() {
         return notes.keySet().iterator();
     }
 
 
-    /**
-     * Release all object references, and initialize instance variables, in preparation for reuse of this object.
-     */
     @Override
     public void recycle() {
 
@@ -919,11 +782,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Remove any object bound to the specified name in the internal notes for this session.
-     *
-     * @param name Name of the note to be removed
-     */
     @Override
     public void removeNote(String name) {
 
@@ -932,9 +790,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Remove a session event listener from this component.
-     */
     @Override
     public void removeSessionListener(SessionListener listener) {
 
@@ -943,13 +798,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Bind an object to a specified name in the internal notes associated with this session, replacing any existing
-     * binding for this name.
-     *
-     * @param name  Name to which the object should be bound
-     * @param value Object to be bound to the specified name
-     */
     @Override
     public void setNote(String name, Object value) {
 
@@ -958,9 +806,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return a string representation of this object.
-     */
     @Override
     public String toString() {
         StringBuilder sb = new StringBuilder();
@@ -1008,11 +853,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     // ------------------------------------------------- HttpSession Properties
 
 
-    /**
-     * Return the time when this session was created, in milliseconds since midnight, January 1, 1970 GMT.
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public long getCreationTime() {
         if (!isValidInternal()) {
@@ -1023,19 +863,12 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return the time when this session was created, in milliseconds since midnight, January 1, 1970 GMT, bypassing the
-     * session validation checks.
-     */
     @Override
     public long getCreationTimeInternal() {
         return this.creationTime;
     }
 
 
-    /**
-     * Return the ServletContext to which this session belongs.
-     */
     @Override
     public ServletContext getServletContext() {
         if (manager == null) {
@@ -1048,14 +881,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
 
     // ----------------------------------------------HttpSession Public Methods
 
-    /**
-     * Return the object bound with the specified name in this session, or <code>null</code> if no object is bound with
-     * that name.
-     *
-     * @param name Name of the attribute to be returned
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public Object getAttribute(String name) {
         if (!isValidInternal()) {
@@ -1070,12 +895,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return an <code>Enumeration</code> of <code>String</code> objects containing the names of the objects bound to
-     * this session.
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public Enumeration<String> getAttributeNames() {
 
@@ -1088,11 +907,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Invalidates this session and unbinds any objects bound to it.
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public void invalidate() {
 
@@ -1106,13 +920,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Return <code>true</code> if the client does not yet know about the session, or if the client chooses not to join
-     * the session. For example, if the server used only cookie-based sessions, and the client has disabled the use of
-     * cookies, then a session would be new on each request.
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public boolean isNew() {
         if (!isValidInternal()) {
@@ -1123,17 +930,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Remove the object bound with the specified name from this session. If the session does not have an object bound
-     * with this name, this method does nothing.
-     * <p>
-     * After this method executes, and if the object implements <code>HttpSessionBindingListener</code>, the container
-     * calls <code>valueUnbound()</code> on the object.
-     *
-     * @param name Name of the object to remove from this session.
-     *
-     * @exception IllegalStateException if this method is called on an invalidated session
-     */
     @Override
     public void removeAttribute(String name) {
 
@@ -1166,20 +962,6 @@ public class StandardSession implements HttpSession, Session, Serializable {
     }
 
 
-    /**
-     * Bind an object to this session, using the specified name. If an object of the same name is already bound to this
-     * session, the object is replaced.
-     * <p>
-     * After this method executes, and if the object implements <code>HttpSessionBindingListener</code>, the container
-     * calls <code>valueBound()</code> on the object.
-     *
-     * @param name  Name to which the object is bound, cannot be null
-     * @param value Object to be bound, cannot be null
-     *
-     * @exception IllegalArgumentException if an attempt is made to add a non-serializable object in an environment
-     *                                         marked distributable.
-     * @exception IllegalStateException    if this method is called on an invalidated session
-     */
     @Override
     public void setAttribute(String name, Object value) {
         setAttribute(name, value, true);
@@ -1356,8 +1138,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
         principal = null; // Transient (may be set later)
         // setId((String) stream.readObject());
         id = (String) stream.readObject();
-        if (manager.getContext().getLogger().isDebugEnabled()) {
-            manager.getContext().getLogger().debug("readObject() loading session " + id);
+        if (manager.getContext().getLogger().isTraceEnabled()) {
+            manager.getContext().getLogger().trace("readObject() loading session " + id);
         }
 
         if (notes == null) {
@@ -1429,8 +1211,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
                 }
                 throw wae;
             }
-            if (manager.getContext().getLogger().isDebugEnabled()) {
-                manager.getContext().getLogger().debug("  loading attribute '" + name + "' with value '" + value + "'");
+            if (manager.getContext().getLogger().isTraceEnabled()) {
+                manager.getContext().getLogger().trace("  loading attribute '" + name + "' with value '" + value + "'");
             }
             // Handle the case where the filter configuration was changed while
             // the web application was stopped.
@@ -1474,8 +1256,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
         stream.writeObject(Boolean.valueOf(isValid));
         stream.writeObject(Long.valueOf(thisAccessedTime));
         stream.writeObject(id);
-        if (manager.getContext().getLogger().isDebugEnabled()) {
-            manager.getContext().getLogger().debug("writeObject() storing session " + id);
+        if (manager.getContext().getLogger().isTraceEnabled()) {
+            manager.getContext().getLogger().trace("writeObject() storing session " + id);
         }
 
         // Gather authentication information (if configured)
@@ -1527,8 +1309,8 @@ public class StandardSession implements HttpSession, Session, Serializable {
             stream.writeObject(saveNames.get(i));
             try {
                 stream.writeObject(saveValues.get(i));
-                if (manager.getContext().getLogger().isDebugEnabled()) {
-                    manager.getContext().getLogger().debug(
+                if (manager.getContext().getLogger().isTraceEnabled()) {
+                    manager.getContext().getLogger().trace(
                             "  storing attribute '" + saveNames.get(i) + "' with value '" + saveValues.get(i) + "'");
                 }
             } catch (NotSerializableException e) {
@@ -1681,19 +1463,4 @@ public class StandardSession implements HttpSession, Session, Serializable {
             }
         }
     }
-
-
-    private static class PrivilegedNewSessionFacade implements PrivilegedAction<StandardSessionFacade> {
-
-        private final HttpSession session;
-
-        PrivilegedNewSessionFacade(HttpSession session) {
-            this.session = session;
-        }
-
-        @Override
-        public StandardSessionFacade run() {
-            return new StandardSessionFacade(session);
-        }
-    }
 }
\ No newline at end of file
