@@ -38,7 +38,6 @@ import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Server;
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.core.StandardContext;
-import org.apache.catalina.security.SecurityConfig;
 import org.apache.juli.ClassLoaderLogManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
@@ -66,6 +65,9 @@ import org.xml.sax.InputSource;
  * <li><b>start</b> - Start an instance of Catalina.</li>
  * <li><b>stop</b> - Stop the currently running instance of Catalina.</li>
  * </ul>
+ *
+ * @author Craig R. McClanahan
+ * @author Remy Maucherat
  */
 public class Catalina {
 
@@ -126,6 +128,12 @@ public class Catalina {
     protected boolean loaded = false;
 
 
+    /**
+     * Rethrow exceptions on init failure.
+     */
+    protected boolean throwOnInitFailure = Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE");
+
+
     /**
      * Generate Tomcat embedded code from configuration files.
      */
@@ -159,7 +167,6 @@ public class Catalina {
     // ----------------------------------------------------------- Constructors
 
     public Catalina() {
-        setSecurityProtection();
         ExceptionUtils.preload();
     }
 
@@ -226,6 +233,24 @@ public class Catalina {
     }
 
 
+    /**
+     * @return <code>true</code> if an exception should be thrown if an error occurs during server init
+     */
+    public boolean getThrowOnInitFailure() {
+        return throwOnInitFailure;
+    }
+
+
+    /**
+     * Set the behavior regarding errors that could occur during server init.
+     *
+     * @param throwOnInitFailure the new flag value
+     */
+    public void setThrowOnInitFailure(boolean throwOnInitFailure) {
+        this.throwOnInitFailure = throwOnInitFailure;
+    }
+
+
     /**
      * Set the shared extensions class loader.
      *
@@ -287,7 +312,7 @@ public class Catalina {
      *
      * @return <code>true</code> if we should continue processing
      */
-    protected boolean arguments(String[] args) {
+    protected boolean arguments(String args[]) {
 
         boolean isConfig = false;
         boolean isGenerateCode = false;
@@ -470,8 +495,8 @@ public class Catalina {
      * Cluster support is optional. The JARs may have been removed.
      */
     private void addClusterRuleSet(Digester digester, String prefix) {
-        Class<?> clazz;
-        Constructor<?> constructor;
+        Class<?> clazz = null;
+        Constructor<?> constructor = null;
         try {
             clazz = Class.forName("org.apache.catalina.ha.ClusterRuleSet");
             constructor = clazz.getConstructor(String.class);
@@ -522,7 +547,7 @@ public class Catalina {
                 Digester.setGeneratedCodeLoader(loader);
             } catch (Exception e) {
                 if (log.isDebugEnabled()) {
-                    log.debug(sm.getString("catalina.noLoader", loaderClassName), e);
+                    log.info(sm.getString("catalina.noLoader", loaderClassName), e);
                 } else {
                     log.info(sm.getString("catalina.noLoader", loaderClassName));
                 }
@@ -640,8 +665,8 @@ public class Catalina {
                         String.valueOf(s.getPortOffset())));
                 log.error(sm.getString("catalina.stopError"), ce);
                 System.exit(1);
-            } catch (IOException ioe) {
-                log.error(sm.getString("catalina.stopError"), ioe);
+            } catch (IOException e) {
+                log.error(sm.getString("catalina.stopError"), e);
                 System.exit(1);
             }
         } else {
@@ -663,8 +688,6 @@ public class Catalina {
 
         long t1 = System.nanoTime();
 
-        initDirs();
-
         // Before digester - it may be needed
         initNaming();
 
@@ -686,7 +709,7 @@ public class Catalina {
         try {
             getServer().init();
         } catch (LifecycleException e) {
-            if (Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE")) {
+            if (throwOnInitFailure) {
                 throw new Error(e);
             } else {
                 log.error(sm.getString("catalina.initError"), e);
@@ -703,7 +726,7 @@ public class Catalina {
     /*
      * Load using arguments
      */
-    public void load(String[] args) {
+    public void load(String args[]) {
 
         try {
             if (arguments(args)) {
@@ -838,14 +861,6 @@ public class Catalina {
     }
 
 
-    /**
-     * @deprecated unused. Will be removed in Tomcat 10 onwards.
-     */
-    @Deprecated
-    protected void initDirs() {
-    }
-
-
     protected void initStreams() {
         // Replace System.out and System.err with a custom PrintStream
         System.setOut(new SystemLogHandler(System.out));
@@ -880,16 +895,6 @@ public class Catalina {
     }
 
 
-    /**
-     * Set the security package access/protection.
-     */
-    protected void setSecurityProtection() {
-        SecurityConfig securityConfig = SecurityConfig.newInstance();
-        securityConfig.setPackageDefinition();
-        securityConfig.setPackageAccess();
-    }
-
-
     protected void generateLoader() {
         String loaderClassName = "DigesterGeneratedCodeLoader";
         StringBuilder code = new StringBuilder();
@@ -908,9 +913,9 @@ public class Catalina {
         File loaderLocation = new File(generatedCodeLocation, generatedCodePackage);
         try (FileWriter writer = new FileWriter(new File(loaderLocation, loaderClassName + ".java"))) {
             writer.write(code.toString());
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             // Should not happen
-            log.debug(sm.getString("catalina.loaderWriteFail"), ioe);
+            log.debug(sm.getString("catalina.loaderWriteFail"), e);
         }
     }
 
@@ -944,7 +949,7 @@ public class Catalina {
 
     // --------------------------------------- CatalinaShutdownHook Inner Class
 
-
+    // XXX Should be moved to embedded !
     /**
      * Shutdown hook which will perform a clean shutdown of Catalina if needed.
      */
@@ -986,7 +991,7 @@ public class Catalina {
 
         }
 
-        ClassLoader parentClassLoader;
+        ClassLoader parentClassLoader = null;
 
         @Override
         public void begin(String namespace, String name, Attributes attributes) throws Exception {
